      Program THREE_D

!        Example of three-dimensional Monte Carlo integration 
!        to find the total charge in a cubic box with exponential distribution
!        rho(x,y,z)=1/(1+x^2) * y*exp(-y^2) * exp(-z)/sqrt(z)

      implicit double precision(a-h,o-z)

      parameter (eps=2.0e-30,iter_max=9,pi=3.1415926)
      parameter (ap=0.0d0,b=10.0,const = pi/2.0d0)

      dimension iseed(10),xi(3)

      data (iseed(j),j=1,9)/234579,90911,1237,343,39,59,10349,511,101/


!        Set EPS as the maximum error allowed.
!        Set ITER_MAX as the maximum number of iterations.
!        Define the integrand as a function $\rho(x,y,z)$.

      rho(xa,ya,za)=(1/(1+xa**2))*(ya*exp(-1.0d0*ya**2))*   &
     &   (exp(-1.0d0*za)/sqrt(za))

!        box normalization

100   a=1.0d0

2     format (A)

      if (a.le.0.0) then
            stop 'Normal end of THREE_D'
       endif

      cnst=a**3
       n=256

!        Calculate the integral for $N$ random points:
!        zero the iteration counter
       iter=0
300    sum_n=0.0

!        initialize sobel sequence
       call sobseq(-1,xi,iseed(iter+1))

       do 390 i=1,n

!        Generate three random numbers with uniform distribution and use
!        them as the values of $x$, $y$, and $z$.

       call sobseq(3,xi,iseed(iter+1))

       x=xi(1)
       y=xi(2)
       z=xi(3)

!        introduce weights

       xp=const *tan((pi/2.0d0)*(x))
       yp = (-1.0d0)*log(y)
       zp= (-1.0d0)*log(z)
       dx=(pi/2.0d0)*const*(1.0d0/cos((pi/2.0d0)*(x))**2)
       dy=(pi)*const*(1.0d0/cos((pi)*(y-.5))**2) 
       dy=(1/y)
       dz=(1/z)

!        Calculate the value of the integrand at the point $(x,y,z)$
!        and add the value to the sum.

       sum_n=sum_n+(rho(xp,yp,zp)*dx*dy*dz)
390    continue

!        Convert the sum to the value of the integral using (2-42) 

       rslt=(cnst*sum_n/float(n))
       print 1,iter,rslt,n
1      format (' Iter',i4,' result =',1pe15.7,'  n',i8)

!        Compare the two values of the integral obtained.

       if (iter.lt.iter_max) then

!        If the difference is larger than $\epsilon$ then
!        take the average of the two values.

       rslt_old=(rslt_old+rslt)/2.0

!        Double the value of $N$
      n=n+n

!        Increase the iteration counter by 1 

      iter=iter+1

!        Repeat the calculation for another N points
      go to 300
      endif
      end

